/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include <map>
#include <unordered_map>
#include "log_manager.h"
#include "storage/disk_manager.h"
#include "system/sm_manager.h"
#include "transaction/transaction.h"
#include "transaction/transaction_manager.h"

class RedoLogsInPage {
   public:
    RedoLogsInPage() { table_file_ = nullptr; }
    RmFileHandle* table_file_;
    std::vector<lsn_t> redo_logs_;  // 在该page上需要redo的操作的lsn
};

class RecoveryManager {
   public:
    RecoveryManager(DiskManager* disk_manager,
                    BufferPoolManager* buffer_pool_manager,
                    SmManager* sm_manager) {
        disk_manager_ = disk_manager;
        buffer_pool_manager_ = buffer_pool_manager;
        sm_manager_ = sm_manager;
    }

    void analyze();
    void redo();
    void undo();
    void redo_log(LogRecord& log_record);
    void redo_insert(const std::string& tab_name, const Rid& rid,
                     const RmRecord& record, Transaction* txn);
    void redo_delete(const std::string& tab_name, const Rid& rid,
                     const RmRecord& record, Transaction* txn);
    void redo_update(const std::string& tab_name, const Rid& rid,
                     const RmRecord& record, const RmRecord& old_record,
                     Transaction* txn);
    lsn_t get_max_active_lsn();
    lsn_t get_persist_lsn() { return persist_lsn_; }
    void finish_job();

   private:
    LogBuffer buffer_;                        // 读入日志
    DiskManager* disk_manager_;               // 用来读写文件
    BufferPoolManager* buffer_pool_manager_;  // 对页面进行读写
    SmManager* sm_manager_;                   // 访问数据库元数据
    lsn_t persist_lsn_;  // 记录已经持久化到磁盘中的最后一条日志的日志号

    // 未完成事务列表（Active Transaction Table-ATT）
    std::unordered_map<txn_id_t, lsn_t> active_txn_;

    // 脏页表（Dirty Page Table-DPT） 映射页面ID到最早未刷新到磁盘的LSN
    std::unordered_map<page_id_t, lsn_t> dirty_page_tab_;

    // 映射lsn到log record
    std::unordered_map<lsn_t, std::shared_ptr<LogRecord>> lsn_log_map_;
};